from PIL import Image
Image.CUBIC = Image.BICUBIC
from tkinter import *
import ttkbootstrap as ttkb
from ttkbootstrap.scrolled import ScrolledText
import fasttext
from iso639 import Lang
from langdetect import detect_langs
from polyglot.detect import Detector
import gcld3

#function called after clicking the button calling all methods
def detect_language():
    text = scrolled_text.get("1.0",'end-1c')
    
    fasttext_result = detect_language_fasttext(text)
    fasttext_label.config(text=f'Fasttext language detection: {Lang(fasttext_result[0]).name}')
    fasttext_meter.configure(amountused=round(fasttext_result[1]*100,2))
    
    langdetect_result = detect_language_langdetect(text)
    langdetect_label.config(text=f'Langdetect language detection: {Lang(list(langdetect_result.keys())[0]).name}')
    langdetect_meter.configure(amountused=round(list(langdetect_result.values())[0]*100,2))
    
    polyglot_result = detect_language_polyglot(text)
    polyglot_label.config(text=f'Polyglot language detection: {Lang(polyglot_result[0]).name}')
    polyglot_meter.configure(amountused=round(polyglot_result[1],2))
    
    gcld3_result = detect_language_gcld3(text)
    gcld3_label.config(text=f'Gcld3 language detection: {Lang(gcld3_result[0]).name}')
    gcld3_meter.configure(amountused=round(gcld3_result[1]*100,2))
    

#fasttext detect language method
def detect_language_fasttext(text):
    try:
        model_path = "lid.176.bin"
        model = fasttext.load_model(model_path)
        lang_detected = model.predict(text)
        return lang_detected[0][0].replace('__label__', ''), lang_detected[1][0]
    except Exception as e:
        print("An error occurred:", e)
        return None

#langdetect detect language method
def detect_language_langdetect(text):
    try:
        x=detect_langs(text)
        lan={}
        for lang in x:
            lan.update({lang.lang: lang.prob})
        return lan
    except Exception as e:
        print("An error occurred:", e)
        return None

#polyglot detect language method
def detect_language_polyglot(text):
    try:
        detector = Detector(text)
        lang_detected = detector.language.code
        confidence = detector.language.confidence
        return lang_detected, confidence
    except Exception as e:
        print("An error occurred:", e)
        return None, 0.0

#gcld3 detect language method
def detect_language_gcld3(text):
    try:
        detector = gcld3.NNetLanguageIdentifier(min_num_bytes=0, max_num_bytes=1000)
        result = detector.FindLanguage(text=text)
        lang_detected = result.language
        lang_probability = result.probability
        print(lang_detected)
        return lang_detected, lang_probability
    except Exception as e:
        print("An error occurred:", e)
        return None


#setup tkinter
root = ttkb.Window(themename="superhero")
root.title("Language Detection Comparison")
root.geometry('1400x900')

#main label
title_label = ttkb.Label(text="Enter text of which language you wish to detect:", font=("Helvetica",28), bootstyle="default")
title_label.pack(pady=30)

#textfield
scrolled_text = ScrolledText(root, height=15, width=120, wrap=WORD, autohide=True, bootstyle="primary")
scrolled_text.pack(pady=20)

#button
confirm_button = ttkb.Button(root, text="Detect & Compare", bootstyle="primary", command=detect_language)
confirm_button.pack(pady=10)

#Results Frame:
frame = LabelFrame(root, padx=10, pady=10, relief="flat")
frame.pack(pady=10)

#labels
fasttext_label = ttkb.Label(frame, text="", font=("Helvetica",14), bootstyle="primary")
fasttext_label.grid(row=0, column=0, padx=20, pady=20)

langdetect_label = ttkb.Label(frame, text="", font=("Helvetica",14), bootstyle="primary")
langdetect_label.grid(row=0, column=1, padx=20, pady=20)

polyglot_label = ttkb.Label(frame, text="", font=("Helvetica",14), bootstyle="primary")
polyglot_label.grid(row=0, column=2, padx=20, pady=20)

gcld3_label = ttkb.Label(frame, text="", font=("Helvetica",14), bootstyle="primary")
gcld3_label.grid(row=0, column=3, padx=20, pady=20)

#meters
fasttext_meter = ttkb.Meter(frame, bootstyle="success",subtext="FastText Probability", textright="%")
fasttext_meter.grid(row=1, column=0, padx=20, pady=20)

langdetect_meter = ttkb.Meter(frame, bootstyle="success",subtext="Langdetect Probability", textright="%")
langdetect_meter.grid(row=1, column=1, padx=20, pady=20)

polyglot_meter = ttkb.Meter(frame, bootstyle="success",subtext="Polyglot Probability", textright="%")
polyglot_meter.grid(row=1, column=2, padx=20, pady=20)

gcld3_meter = ttkb.Meter(frame, bootstyle="success",subtext="Gcld3 Probability", textright="%")
gcld3_meter.grid(row=1, column=3, padx=20, pady=20)

root.mainloop()